@@ -32,6 +32,7 @@
     (define-key map (kbd "<C-left>")      'helm-ucs-persistent-backward)
     (define-key map (kbd "<C-right>")     'helm-ucs-persistent-forward)
     (define-key map (kbd "<C-return>")    'helm-ucs-persistent-insert)
+    (define-key map (kbd "C-c SPC")       'helm-ucs-persistent-insert-space)
     map)
   "Keymap for `helm-ucs'.")
 
@@ -75,13 +76,82 @@
 (defvar helm-ucs--names nil)
 (defvar helm-ucs-history nil)
 
-(defun helm-calculate-ucs-max-len ()
-  "Calculate the length of longest `ucs-names' candidate."
-  (cl-loop for (_n . v) in (ucs-names)
+(defun helm-calculate-ucs-alist-max-len (names)
+  "Calculate the length of the longest NAMES list candidate."
+  (cl-loop for (_n . v) in names
            maximize (length (format "#x%x:" v)) into code
            maximize (max 1 (string-width (format "%c" v))) into char
            finally return (cons code char)))
 
+(defun helm-calculate-ucs-hash-table-max-len (names)
+  "Calculate the length of the longest NAMES hash table candidate."
+  (cl-loop for _n being the hash-keys of names
+           using (hash-values v)
+           maximize (length (format "#x%x:" v)) into code
+           maximize (max 1 (string-width (format "%c" v))) into char
+           finally return (cons code char)))
+
+(defun helm-calculate-ucs-max-len ()
+  "Calculate the length of longest `ucs-names' candidate."
+  (let ((ucs-struct (ucs-names)))
+    (if (hash-table-p ucs-struct)
+        (helm-calculate-ucs-hash-table-max-len ucs-struct)
+      (helm-calculate-ucs-alist-max-len ucs-struct))))
+
+(defun helm-ucs-collect-symbols-alist (names)
+  "Collect ucs symbols from the NAMES list."
+  (cl-loop with pr = (make-progress-reporter
+                      "collecting ucs names"
+                      0 (length names))
+           for (n . v) in names
+           for count from 1
+           for xcode = (format "#x%x:" v)
+           for len = (length xcode)
+           for diff = (- (car helm-ucs--max-len) len)
+           for code = (format "(#x%x): " v)
+           for char = (propertize (format "%c" v)
+                                  'face 'helm-ucs-char)
+           unless (or (string= "" n)
+                      ;; `char-displayable-p' return a font object or
+                      ;; t for some char that are displayable but have
+                      ;; no special font (e.g 10) so filter out char
+                      ;; with no font.
+                      (not (fontp (char-displayable-p (read xcode)))))
+           collect
+           (concat code (make-string diff ? )
+                   char "  " n)
+           and do (progress-reporter-update pr count)))
+
+(defun helm-ucs-collect-symbols-hash-table (names)
+  "Collect ucs symbols from the NAMES hash-table."
+  (cl-loop with pr = (make-progress-reporter
+                      "collecting ucs names"
+                      0 (hash-table-count names))
+           for n being the hash-keys of names
+           using (hash-values v)
+           for count from 1
+           for xcode = (format "#x%x:" v)
+           for len = (length xcode)
+           for diff = (- (car helm-ucs--max-len) len)
+           for code = (format "(#x%x): " v)
+           for char = (propertize (format "%c" v)
+                                  'face 'helm-ucs-char)
+           unless (or (string= "" n)
+                      (not (fontp (char-displayable-p (read xcode)))))
+           collect
+           (concat code (make-string diff ? )
+                   char "  " n)
+           and do (progress-reporter-update pr count)))
+
+(defun helm-ucs-collect-symbols (ucs-struct)
+  "Collect ucs symbols from UCS-STRUCT.
+
+Depending on the Emacs version, the variable `ucs-names' can
+either be an alist or a hash-table."
+  (if (hash-table-p ucs-struct)
+      (helm-ucs-collect-symbols-hash-table ucs-struct)
+    (helm-ucs-collect-symbols-alist ucs-struct)))
+
 (defun helm-ucs-init ()
   "Initialize an helm buffer with ucs symbols.
 Only math* symbols are collected."
@@ -90,15 +160,57 @@ Only math* symbols are collected."
           (helm-calculate-ucs-max-len)))
   (or helm-ucs--names
       (setq helm-ucs--names
-            (cl-loop for (n . v) in (ucs-names)
-                     for len = (length (format "#x%x:" v))
-                     for diff = (- (car helm-ucs--max-len) len)
-                     for code = (format "(#x%x): " v)
-                     for char = (propertize (format "%c" v)
-                                            'face 'helm-ucs-char)
-                     unless (string= "" n) collect
-                     (concat code (make-string diff ? )
-                             char "  " n)))))
+            (helm-ucs-collect-symbols (ucs-names)))))
+
+;; Actions (insertion)
+
+(defun helm-ucs-match (candidate n)
+  "Return the N part of an ucs CANDIDATE.
+Where N=1 is the ucs code, N=2 the ucs char and N=3 the ucs name."
+  (when (string-match
+         "^(\\(#x[a-f0-9]+\\)): *\\(.\\) *\\([^:]+\\)+"
+         candidate)
+    (match-string n candidate)))
+
+(defun helm-ucs-insert (candidate n)
+  "Insert the N part of CANDIDATE."
+  (with-helm-current-buffer
+    (insert (helm-ucs-match candidate n))))
+
+(defun helm-ucs-insert-char (candidate)
+  "Insert ucs char part of CANDIDATE at point."
+  (helm-ucs-insert candidate 2))
+
+(defun helm-ucs-insert-code (candidate)
+  "Insert ucs code part of CANDIDATE at point."
+  (helm-ucs-insert candidate 1))
+
+(defun helm-ucs-insert-name (candidate)
+  "Insert ucs name part of CANDIDATE at point."
+  (helm-ucs-insert candidate 3))
+
+(defun helm-ucs-persistent-insert ()
+  "Insert ucs char without quitting helm."
+  (interactive)
+  (with-helm-alive-p
+    (helm-attrset 'action-insert 'helm-ucs-insert-char)
+    (helm-execute-persistent-action 'action-insert)))
+(put 'helm-ucs-persistent-insert 'helm-only t)
+
+;; Kill actions
+(defun helm-ucs-kill-char (_candidate)
+  "Action that concatenate ucs marked chars."
+  (kill-new (mapconcat (lambda (x)
+                         (helm-ucs-match x 2))
+                       (helm-marked-candidates) "")))
+
+(defun helm-ucs-kill-code (candidate)
+  (kill-new (helm-ucs-match candidate 1)))
+
+(defun helm-ucs-kill-name (candidate)
+  (kill-new (helm-ucs-match candidate 3)))
+
+;; Navigation in current-buffer (persistent)
 
 (defun helm-ucs-forward-char (_candidate)
   (with-helm-current-buffer
@@ -112,28 +224,9 @@ Only math* symbols are collected."
   (with-helm-current-buffer
     (delete-char -1)))
 
-(defun helm-ucs-insert (candidate n)
-  (when (string-match
-         "^(\\(#x[a-f0-9]+\\)): *\\(.\\) *\\([^:]+\\)+"
-         candidate)
-    (with-helm-current-buffer
-      (insert (match-string n candidate)))))
-
-(defun helm-ucs-insert-char (candidate)
-  (helm-ucs-insert candidate 2))
-
-(defun helm-ucs-insert-code (candidate)
-  (helm-ucs-insert candidate 1))
-
-(defun helm-ucs-insert-name (candidate)
-  (helm-ucs-insert candidate 3))
-
-(defun helm-ucs-persistent-insert ()
-  (interactive)
-  (with-helm-alive-p
-    (helm-attrset 'action-insert 'helm-ucs-insert-char)
-    (helm-execute-persistent-action 'action-insert)))
-(put 'helm-ucs-persistent-insert 'helm-only t)
+(defun helm-ucs-insert-space (_candidate)
+  (with-helm-current-buffer
+    (insert " ")))
 
 (defun helm-ucs-persistent-forward ()
   (interactive)
@@ -156,6 +249,12 @@ Only math* symbols are collected."
     (helm-execute-persistent-action 'action-delete)))
 (put 'helm-ucs-persistent-delete 'helm-only t)
 
+(defun helm-ucs-persistent-insert-space ()
+  (interactive)
+  (with-helm-alive-p
+    (helm-attrset 'action-insert-space 'helm-ucs-insert-space)
+    (helm-execute-persistent-action 'action-insert-space)))
+
 (defvar helm-source-ucs
   (helm-build-in-buffer-source "Ucs names"
     :data #'helm-ucs-init
@@ -167,9 +266,10 @@ Only math* symbols are collected."
     :action '(("Insert character" . helm-ucs-insert-char)
               ("Insert character name" . helm-ucs-insert-name)
               ("Insert character code in hex" . helm-ucs-insert-code)
-              ("Forward char" . helm-ucs-forward-char)
-              ("Backward char" . helm-ucs-backward-char)
-              ("Delete char backward" . helm-ucs-delete-backward)))
+              ("Kill marked characters" . helm-ucs-kill-char)
+              ("Kill name" . helm-ucs-kill-name)
+              ("Kill code" . helm-ucs-kill-code))
+    :keymap  helm-ucs-map)
   "Source for collecting `ucs-names' math symbols.")
 
 ;;;###autoload
@@ -180,12 +280,17 @@ Only math* symbols are collected."
         :buffer "*helm select xfont*"))
 
 ;;;###autoload
-(defun helm-ucs ()
-  "Preconfigured helm for `ucs-names' math symbols."
-  (interactive)
+(defun helm-ucs (arg)
+  "Preconfigured helm for `ucs-names'.
+
+Called with a prefix arg force reloading cache."
+  (interactive "P")
+  (when arg
+    (setq helm-ucs--names nil
+          helm-ucs--max-len nil
+          ucs-names nil))
   (let ((char (helm-aif (char-after) (string it))))
     (helm :sources 'helm-source-ucs
-          :keymap  helm-ucs-map
           :history 'helm-ucs-history
           :input (and char (multibyte-string-p char) char)
           :buffer "*helm ucs*")))
