@@ -242,7 +242,9 @@ This is a command for `helm-kill-ring-map'."
 
 (defun helm-mark-ring-get-candidates ()
   (with-helm-current-buffer
-    (cl-loop with marks = (if (mark t) (cons (mark-marker) mark-ring) mark-ring)
+    (cl-loop with marks = (if (mark t)
+                              (cons (mark-marker) mark-ring)
+                            mark-ring)
              for marker in marks
              with max-line-number = (line-number-at-pos (point-max))
              with width = (length (number-to-string max-line-number))
@@ -255,13 +257,19 @@ This is a command for `helm-kill-ring-map'."
 
 (defun helm-mark-ring-default-action (candidate)
   (let ((target (copy-marker candidate)))
-    (switch-to-buffer (marker-buffer candidate))
-    (helm-log-run-hook 'helm-goto-line-before-hook)
-    (helm-match-line-cleanup)
-    (with-helm-current-buffer
-      (unless helm-yank-point (setq helm-yank-point (point))))
-    (helm-goto-char target)
-    (helm-highlight-current-line)))
+    (helm-aif (marker-buffer candidate)
+        (progn
+          (switch-to-buffer it)
+          (helm-log-run-hook 'helm-goto-line-before-hook)
+          (helm-match-line-cleanup)
+          (with-helm-current-buffer
+            (unless helm-yank-point (setq helm-yank-point (point))))
+          (helm-goto-char target)
+          (helm-highlight-current-line))
+      ;; marker points to no buffer, no need to dereference it, just
+      ;; delete it.
+      (setq mark-ring (delete target mark-ring))
+      (error "Marker points to no buffer"))))
 
 (defvar helm-source-mark-ring
   (helm-build-sync-source "mark-ring"
