@@ -160,10 +160,13 @@ you have completion on these functions with `C-M i' in the customize interface."
   "Preconfigured `helm' for `semantic'.
 If ARG is supplied, pre-select symbol at point instead of current"
   (interactive "P")
-  (let ((tag (helm-aif (semantic-current-tag-parent)
-                  (cons (format "\\_<%s\\_>" (car it))
-                        (format "\\_<%s\\_>" (car (semantic-current-tag))))
-                (format "\\_<%s\\_>" (car (semantic-current-tag))))))
+  (let ((tag (helm-aif (car (semantic-current-tag-parent))
+                 (let ((curtag (car (semantic-current-tag))))
+                   (if (string= it curtag)
+                       (format "\\_<%s\\_>" curtag)
+                     (cons (format "\\_<%s\\_>" it)
+                           (format "\\_<%s\\_>" curtag))))
+               (format "\\_<%s\\_>" (car (semantic-current-tag))))))
     (unless helm-source-semantic
       (setq helm-source-semantic
             (helm-make-source "Semantic Tags" 'helm-semantic-source
@@ -202,9 +205,12 @@ Fill in the symbol at point by default."
          (helm-execute-action-at-once-if-one
           (and imenu-p
                helm-imenu-execute-action-at-once-if-one))
-         (tag (helm-aif (semantic-current-tag-parent)
-                  (cons (format "\\_<%s\\_>" (car it))
-                        (format "\\_<%s\\_>" (car (semantic-current-tag))))
+         (tag (helm-aif (car (semantic-current-tag-parent))
+                  (let ((curtag (car (semantic-current-tag))))
+                    (if (string= it curtag)
+                        (format "\\_<%s\\_>" curtag)
+                      (cons (format "\\_<%s\\_>" it)
+                            (format "\\_<%s\\_>" curtag))))
                 (format "\\_<%s\\_>" (car (semantic-current-tag))))))
     (helm :sources source
           :candidate-number-limit 9999
